#ifndef _GC_VM_H
#define _GC_VM_H

/*
 * The garbage collector's interface to the VM
 */

/*
 * Enumerate root locations for a collection.  The VM must call
 * addRootLocation on each address that holds a pointer into the heap
 * from outside.
 */
void enumerateRoots();

/*
 * Object scanning.
 *
 * Return the number of pointer fields in the given object.
 */
int referenceFieldCount(ObjectReference obj);

/*
 * Return an array of the offsets of the pointers in an object
 */
int[] referenceFieldOffsets(ObjectReference obj);

/*
 * Return the size of an object, including header etc, once the
 * object has been copied, ie allowing for increasing size due to
 * address-based hashing etc
 */
int getSizeWhenCopied(ObjectReference obj);

/*
 * Copy an object.  Gives the VM a chance to install a hash word and that kind of 
 * stuff.
 */
void copyObject(Address newObject, Address oldObject);

/*
 * Basic facts about an object
 */
int isArray(ObjectReference object);
int hasPointers(ObjectReference object);