#ifndef OBJECT_MODEL_H_
#define OBJECT_MODEL_H_

/*
 * Define the header of a heap object.
 * 
 * This is a two-word header
 * 
 * +--------------------------------+
 * |llllllllllllllllllllllllhha   gg|
 * +--------------------------------+
 * |       TIB pointer              |
 * +--------------------------------+
 *
 * Where l = lock word
 *       h = hash status bits.  If hashed_moved is set, there is an additional
 *           field at the end of the object containing the value of the
 *           address the object occupied when it was hashed.
 *       a = Is this an array or a scalar
 *       g = GC bits, use depends on the collector.
 * 
 * For an array, the following word is the length.  For a scalar, it is
 * the first field of the object.
 * 
 * 
 */
typedef void * ObjectReference;

typedef struct _object_header {
	int lock:24;            // Thin lock
	int hashed:1;           // Has the object been hashed ?
	int hashed_moved:1;     // Has it been moved since being hashed ?
    int isArray:1;          // 1 if it's an array
    int hasPointersy:1;     // 1 if this object has pointers
	int _unused:2;
	int gcbits:2;           // GC status bits
	void *tib;              // Per-class info pointer
	union {
	  int length;             // Array length or first field of scalar
	  ObjectReference fwptr;  // Forwarding pointer
	};
} object_header;

#define HEADER_OFFSET (-sizeof(object_header))

__inline__ 
char * objectReferenceToAddress(char *ref) {
	return ref + HEADER_OFFSET;
}

__inline__
char * addressToObjectReference(char *addr) {
	return addr - HEADER_OFFSET;
}
#endif /*OBJECT_MODEL_H_*/
